package au.csiro.cmis.sact.xmlblaster;

import org.xml.sax.ContentHandler;
import org.xml.sax.ErrorHandler;
import org.xmlBlaster.client.I_Callback;
import org.xmlBlaster.client.SubscribeKeyWrapper;
import org.xmlBlaster.client.SubscribeQosWrapper;
import org.xmlBlaster.client.UpdateKey;
import org.xmlBlaster.client.UpdateQos;
import org.xmlBlaster.client.protocol.XmlBlasterConnection;
import org.xmlBlaster.util.XmlBlasterException;

/**
 * <p>Title: Spread Benchmarks</p>
 * <p>Description: Test spread (http://www.spread.org) performance</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CSIRO</p>
 * @author Doug Palmer &lt;Doug.Palmer@csiro.au&gt;
 * @version 1.0
 */

public class Receiver implements I_Callback {
  // The listener for this
  private class ReceiverListener implements I_Callback {
    public String update(String cbSessionId, UpdateKey key, byte[] content, UpdateQos qos) {
      if (Receiver.this.counting)
        Receiver.this.count++;
      return "";
    }
  }

  // The name of the sender
  private String name;
  // The group to join
  private String groupName;
  // The quality of service
  private String qos;
  // The topic type to use
  private String topicType;
  // The spread connection
  private XmlBlasterConnection connection;
 // The xmlBlaster subscribe key
  private String subscribeKey;
  // The xmlBlaster publish qos
  private SubscribeQosWrapper subscribeQos;
  // The subscription id
  private String subscribeOid;
  // The listener
  private ReceiverListener listener;
  // The message count
  private int count;
  // Should we be counting
  private boolean counting;

  /**
   * Constructor
   *
   * @param name The name of the sender
   * @param groupName The group name to send to
   * @param server The hostname of the daemon to connect to
   * @param port The port number to connect to
   * @param topicType The way to handle topics
   */
  public Receiver(String name, String groupName, String qos, String topicType) {
    this.name = name;
    this.groupName = groupName;
    this.qos = qos;
    this.topicType = topicType;
    this.count = 0;
    this.counting = false;
  }

  /**
   * Connect this sender to the spread daemon
   */
  public void connect() throws XmlBlasterException {
    if (this.connection == null) {
      this.listener = new ReceiverListener();
      this.connection = new XmlBlasterConnection();
      this.connection.connect(null, this); // Why do I have to listen to it?
      if (this.topicType.equalsIgnoreCase("xpath"))
        this.subscribeKey =
          "<key oid='' queryType='XPATH'>" +
          "/xmlBlaster/key/" + this.groupName +
          "</key>";
      else
        this.subscribeKey =
          "<key oid='" + this.groupName + "' queryType='EXACT'>" +
          "</key>";
      this.subscribeQos = new SubscribeQosWrapper();
      this.subscribeOid = null;
    }
  }

  /**
   * Disconnect from the spread daemon
   */
  public void disconnect() throws XmlBlasterException {
    if (this.connection != null) {
      this.connection.disconnect(null);
      this.connection.shutdown();
      this.connection = null;
    }
  }

  /**
   * Start receiving messages
   */
  public void start() throws XmlBlasterException {
    this.count = 0;
    this.counting = true;
    this.subscribeOid = this.connection.subscribe(this.subscribeKey, this.subscribeQos.toXml(), this.listener).getSubscriptionId();
  }

  /**
   * Stop counting
   */
  public void idle() {
    this.counting = false;
  }

  /**
   * Stop receiving messages
   */
  public void stop() throws XmlBlasterException {
    SubscribeKeyWrapper unsubKey = new SubscribeKeyWrapper(this.subscribeOid);

    this.connection.unSubscribe(unsubKey.toXml(), this.subscribeQos.toXml());
    this.subscribeOid = null;
    this.counting = false;
  }

  /**
   * Get the count of messages
   *
   * @return The number of messages received
   */
  public int getCount() {
    return this.count;
  }

  /**
   * The default callback for this subscription.
   * <P>
   * If this is called, then there is an error afoot.
   * However, xmlBlaster seems to send all messages to everyone.
   * No matter what's subscribed.
   */
  public String update(String cbSessionId, UpdateKey key, byte[] content, UpdateQos qos) {
      System.err.println("Receieved unsubscribed message in session " + cbSessionId + " key=" + key.toString());
      return "";
  }
}